#!/usr/bin/env python2

#######################################################
#
#    Winzig's Calculator
#
#    Copyright (C) 2001-2003 Robert Muth <robert at muth dot org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of June 1991.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program in form of the file COPYING; 
#    if not, write to the 
#
#    Free Software Foundation, Inc. <http://www.fsf.org>
#    59 Temple Place, Suite 330, 
#    Boston, MA 02111-1307  USA
#    $Id: wcalculator.py,v 1.14 2003/11/23 19:16:41 muth Exp $
#
#######################################################

from __future__ import nested_scopes

import gtk
import gobject

import time 
import re 
import string
import math

import wconfig
from wutil import *

#######################################################
## 
#######################################################

patOperation = re.compile("^[\~\*\+\-\/\%\&\|\@\^]")
patVariable = re.compile("^\$[0-9]+")
patFun = re.compile("^[a-zA-Z]+")
patNumber = re.compile("^[xX0-9a-fA-F\.]+")
patBracket = re.compile("^[\(\)]")
patWhitespace = re.compile("^\s+")

#######################################################

tag_value  = "v"
tag_binary_op = "b" 
tag_unary_op = "u"
tag_paren_open = "("
tag_paren_close = ")"

class classOperator:

    def __init__(self,op,prec,desc):
        self.operation = op
        self.precedence = prec
        self.description = desc                


def operator_id(x):
    return x

def operator_not(x):
    return -1 ^ x

def operator_neg(x):
    return -x

def operator_add(x,y):
    return x + y

def operator_sub(x,y):
    return x - y

def operator_mul(x,y):
    return x * y

def operator_div(x,y):
    return x / y

def operator_mod(x,y):
    return x % y

def operator_pow(x,y):
    return x ** y

#######################################################

Binary = {}
Binary["+"] = classOperator( operator_add,  6, "addition")
Binary["-"] = classOperator( operator_sub,  6, "substraction")
Binary["*"] = classOperator( operator_mul,  5, "multiplication")
Binary["/"] = classOperator( operator_div,  5, "division")
Binary["%"] = classOperator( operator_mod,  5, "modulo")
Binary["^"] = classOperator( operator_pow,  4, "power (left assoc.)")
#Binary["|"] = classOperator( operator.or_,  3, "bitwise or")
#Binary["&"] = classOperator( operator.and_, 2, "bitwise and")
#Binary["@"] = classOperator( operator.xor,  2, "bitwise xor")

#######################################################

Unary = {}
Unary["+"] =    classOperator( operator_id, 0,  "unary plus")
Unary["-"] =    classOperator( operator_neg, 0, "unary minus") 
Unary["~"] =    classOperator( operator_not, 0, "bitwise negation")
Unary["sin"] =  classOperator( math.sin, 0,     "sin")
Unary["cos"] =  classOperator( math.cos, 0,     "cos")
Unary["tan"] =  classOperator( math.tan, 0,     "tan")
Unary["asin"] = classOperator( math.asin, 0,    "arcsin")
Unary["acos"] = classOperator( math.acos, 0,    "arccos")
Unary["atan"] = classOperator( math.atan, 0,    "arctan")
Unary["exp"] =  classOperator( math.exp, 0,     "exp (base e)")      
Unary["log"] =  classOperator( math.log, 0,     "log (base e)")
Unary["sqrt"] = classOperator( math.sqrt,0,     "sqrt")  

#######################################################

class SyntaxError(Exception):
    def __init__(self, message):
        self._message = message

    def __str__(self):
        return str(self._message)

    def Message(self):
        return self._message
    
#######################################################

def Coerce( value, mode ):
    if mode == "float":
        return float(value)
    elif mode == "int32":
        return int(value)
    elif mode == "long":
        return long(value)
#    elif mode == "COMPLEX":
#        return complex(value)
    else:
        raise SyntaxError("bad mode " + mode )

#######################################################

def MakeValue(value,mode):
    return [tag_value,Coerce(value,mode)]

def MakeBinaryOp(op):
    return [tag_binary_op,op,Binary[op].precedence]

def OpIsNotBinary(op):
    return not Binary.has_key(op)
    
def MakeUnaryOp(op):
    return [tag_unary_op,op]

def MakeParenOpen():
    return [tag_paren_open]

def MakeParenClose():
    return [tag_paren_close]

#######################################################

def GetValue(list):
    return list[1]

def GetPriority(list):
    return list[2]

def GetOperator(list):
    return list[1]

def GetTag(list):
    return list[0]

def GetTags(lists):
    return map(GetTag,lists)

#######################################################

def Tokenize(expression,mode,value_stack):
     tokens = []

     # we surrond the whole expression by a pair of parenthesis
     
     tokens.append( MakeParenOpen())
     next_token_could_be_unary = 1
     
     while expression != "":
         # print "tokenize:  " + expression + "   ", tokens

         match = patOperation.search(expression)
         if match != None:
             token = match.group(0)
             if next_token_could_be_unary:
                 tokens.append(MakeUnaryOp(token))
             else:
                 if OpIsNotBinary(token): raise SyntaxError("token is not binary op " + token)
                 tokens.append(MakeBinaryOp(token))
             # print "match: " + match.group(0) + "  end: " + "%d" % match.end()
             expression = expression[match.end():]
             next_token_could_be_unary = 1
             continue             

         match = patVariable.search(expression)
         if match != None:
             token = match.group(0)
             var = token[1:]
             index = int(var)
             if index >= len(value_stack):
                 raise SyntaxError("unknown variable " + token)
             val = MakeValue(value_stack[index],mode)
             tokens.append( val )
             # print "match: " + match.group(0) + "  end: " + "%d" % match.end()
             expression = expression[match.end():]
             next_token_could_be_unary = 0
             continue       

         match = patFun.search(expression)
         if match != None:
             token = match.group(0)
             if next_token_could_be_unary:
                 tokens.append(MakeUnaryOp(token))
             else:
                raise SyntaxError("misplaced function " + token)
                return None
             # print "match: " + match.group(0) + "  end: " + "%d" % match.end()
             expression = expression[match.end():]
             next_token_could_be_unary = 1
             continue             

         match = patNumber.search(expression)
         if match != None:
             token =  match.group(0)
             try: 
                 val =  MakeValue( string.atoi(token,0),mode)
             except:
                 try:
                     val = MakeValue(float(token),mode)
                 except:
                     raise SyntaxError("malformed number " + token)
                         
             tokens.append( val )
             # print "match: " + match.group(0) + "  end: " + "%d" % match.end()
             expression = expression[match.end():]
             next_token_could_be_unary = 0
             continue              

         match = patBracket.search(expression)
         if match != None:
             token = match.group(0)
             # print "match: " + match.group(0) + "  end: " + "%d" % match.end()
             expression = expression[match.end():]
             if token == "(":
                tokens.append( MakeParenOpen())
                next_token_could_be_unary = 1
             else:
                tokens.append(MakeParenClose())
                next_token_could_be_unary = 0
             continue      

         match = patWhitespace.search(expression)
         if match != None:
             expression = expression[match.end():]
             continue

         raise SyntaxError("unknown expression " + expression)

     tokens.append(MakeParenClose())
     return tokens
 

#######################################################
##
#######################################################

def BinaryCompute(op,val1,val2):
    if not Binary.has_key(op):
        raise SyntaxError("unknown binary op " + op)
    
    fun = Binary[op].operation
       
    return fun(val1,val2 )

#######################################################

def UnaryCompute(op,val):
    if not Unary.has_key(op):
        raise SyntaxError("unknown unary op " + op)

    fun = Unary[op].operation

    return fun(val)

#######################################################
##
#######################################################

def TryToReduceStack(stack,mode):

    # we are reducing at the end of the stack

    if GetTags(stack[-3:]) == [tag_paren_open, tag_value, tag_paren_close]:
        #Debug("REDUCE (V)  -> V\n")
        val = stack[-2]
        stack = stack[:-3] 
        stack.append(val)    
        return (stack,1)

    if GetTags(stack[-4:]) == [tag_value, tag_binary_op, tag_value, tag_paren_close]:
        #Debug("REDUCE VBV) -> V\'\n")
        top = stack[-1]
        val = BinaryCompute(GetOperator(stack[-3]),GetValue(stack[-4]),GetValue(stack[-2]));
        stack = stack[:-4] 
        stack.append(MakeValue(val,mode))    
        stack.append(top)    
        return (stack,1)

    if GetTags(stack[-2:]) == [tag_unary_op, tag_value]:
        #Debug("REDUCE UV -> V\'\n")
        val = UnaryCompute(GetOperator(stack[-2]),GetValue(stack[-1]));
        stack = stack[:-2]
        stack.append(MakeValue(val,mode))    
        return (stack,1)

    if GetTags(stack[-4:]) == [tag_value, tag_binary_op, tag_value, tag_binary_op]:
        if GetPriority(stack[-3]) <= GetPriority(stack[-1]):    
            #Debug("REDUCE VBVB\' -> V\'B\n")
            top = stack[-1]
            val = BinaryCompute(GetOperator(stack[-3]),GetValue(stack[-4]),GetValue(stack[-2]));
            stack = stack[:-4]
            stack.append(MakeValue(val,mode))    
            stack.append(top)    
            return (stack,1)

    return (stack,0)

#######################################################
#######################################################

type_int = type(1)
type_float = type(1.0)
type_long = type(1L)
type_complex = type(1.0j)
type_list  = type([])

#######################################################
##
## 
#######################################################

def EvaluateExpression(tokens,mode):

    stack = []

    # we are reducing at the end of the stack
    for t in tokens:
        stack.append(t)
        while len(stack) != 1:
            #Debug("Stack: " + str(stack) + "\n")
            stack, change =  TryToReduceStack(stack,mode)
            if not change: break

    #Debug("Final: " + str(stack) + "\n")

    if len(stack) != 1 or GetTag(stack[-1]) != tag_value:
        raise SyntaxError("not a well formed expression")
    else:
        return GetValue(stack[-1])

#######################################################
##
#######################################################

class classGui:
    def __init__(self): 
        self.calc = None
        self.help  = None
        self.count = 0
        self.value_history = []
        self.list = []

#######################################################
#######################################################

def ActionSwitchToHelp(args,gui):
    gui.calc.hide()
    gui.help.show()
    return
#######################################################
#######################################################

def ActionSwitchToCalc(args,gui):

    gui.help.hide()
    gui.calc.show()


#######################################################
##
#######################################################
def CreateWidgetHelp(gui,tooltips):

    buffer = "<big>"
    buffer = buffer + "Formats are printf style (%s is generic)\n"
    buffer = buffer + "Modes reflect python numeric types\n"
    buffer = buffer + "Expressions are infix!\n"
    buffer = buffer + "Double clicking on values adds them to expression!\n"
    buffer = buffer + "</big>\n"



    buffer = buffer + "<big>Binary Operators (with precedence):</big>\n<tt>"
    for k in Binary.keys():
        line = ("%-5s" % k) + (" %d " % Binary[k].precedence) + ("%-20s" % Binary[k].description) + "\n"
        buffer = buffer + line
    buffer = buffer + "</tt>\n"

    buffer = buffer + "<big>Unary Operators:</big>\n<tt>"
    for k in Unary.keys():
        line = ("%-5s" % k) + ("   ") + ("%-20s" % Unary[k].description) + "\n"
        buffer = buffer + line
    buffer = buffer + "</tt>"

     
#    print buffer
    vbox = gtk.VBox()
    vbox.show()

    hbox = gtk.HBox()
    vbox.pack_start(hbox,0,0)
    hbox.show()

    button = gtk.Button(" Done ")
    hbox.pack_start(button,0,0,0)
    button.connect('clicked', ActionSwitchToCalc, gui)
    button.show()
 
    window = gtk.ScrolledWindow()
    window.set_policy(gtk.POLICY_NEVER,gtk.POLICY_ALWAYS)
    vbox.pack_start(window)
    window.show() 

    text = gtk.Label("")
    text.set_markup(buffer)
    text.show()

    window.add_with_viewport(text)

    return vbox

#######################################################
##
#######################################################

def CreateWidgetCalc(gui,tooltips):

    vbox = gtk.VBox()
    vbox.show()

    ##

    hbox = gtk.HBox()
    vbox.pack_start(hbox,0,0)
    hbox.show()
    
    button = gtk.Button(" Help ")
    hbox.pack_start(button,0,0,0)
    button.connect('clicked', ActionSwitchToHelp,gui)
    button.show()
    tooltips.set_tip( button,"Show operations");


    mode = gtk.Combo()
    mode.set_popdown_strings(["float","int32","long"])
    hbox.pack_end(mode,0,0,0)
    mode.entry.set_editable(0)

    mode.entry.set_width_chars(6)
    mode.show()
    tooltips.set_tip( mode.entry,"select mode of operation");

    label =  gtk.Label(" Mode:")
    hbox.pack_end(label,0,0,0)
    label.show()

    format = gtk.Combo()
    format.set_popdown_strings(["%s","%d","0x%x","%f","%e","%g"])
    format.disable_activate()
    format.entry.set_width_chars(5)


    hbox.pack_end(format,0,0,0)
    format.show()
    tooltips.set_tip( format.entry,"select value formating");

    label =  gtk.Label(" Fmt:")
    hbox.pack_end(label,0,0,0)
    label.show()

    #########################################################
    # FORWARD DECLARATION FOR READABILITY
    #########################################################
    expression = None
    #########################################################

    def ActionBackSpace(args):
        entry =  expression.entry
        pos =  entry.get_position()
        entry.delete_text(pos-1,pos)
        return
    
    def ActionAddToInput(args,str):

        if args:
            if "0" <= str and str <= "9" or str == "." or str == "x":
                pass
            elif "$" == str:
                str = " " + str            
            else:
                str = " " + str + " "
        entry =  expression.entry
        entry.set_position( entry.insert_text(str, entry.get_position()))
        return

    #########################################################

    window = gtk.ScrolledWindow()
    window.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_ALWAYS)
    window.set_shadow_type (gtk.SHADOW_ETCHED_IN);
    vbox.pack_start(window)
    window.show() 

    gui.vadj = window.get_vadjustment()


    def ActionHistoryClicked(a,row,c):
        ActionAddToInput(None,r" $%d" % row[0])
        return
    
    model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)

    gui.model = model

    view = gtk.TreeView(model)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("   Value    ", renderer, text=0)
#    column.set_sort_column_id(1)
    column.set_resizable(1)
    view.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("   Expression   ", renderer, text=1)
#    column.set_sort_column_id(2)
    column.set_resizable(1)
    view.append_column(column)

    
    view.connect("row_activated",ActionHistoryClicked)
    view.set_search_column(1);

    view.show()

    window.add_with_viewport(view)


    hbox = gtk.HBox()
    vbox.pack_start(hbox,0,0)
    hbox.show()

    buttons = ["0","1","2","3","4","5","6","7","8","9"]
    for b in buttons:
        button = gtk.Button(b)
        hbox.pack_start(button,1,1,0)
        button.connect('clicked', ActionAddToInput,b)
        button.show()

    button = gtk.Button(" BS ")
    hbox.pack_start(button,1,1,0)
    button.connect('clicked', ActionBackSpace)
    button.show()
        
    hbox = gtk.HBox()
    vbox.pack_start(hbox,0,0)
    hbox.show()

    buttons = ["+","-","*","/","^","%","(","$",")","~",".","x"]
    for b in buttons:
        button = gtk.Button(b)
        hbox.pack_start(button,1,1,0)
        button.connect('clicked', ActionAddToInput,b)
        button.show()

    hbox = gtk.HBox()
    vbox.pack_start(hbox,0,0)
    hbox.show()

    buttons = ["log","exp","sqrt","sin","cos","tan","asin","acos","atan"]
    for b in buttons:
        button = gtk.Button(b)
        hbox.pack_start(button,1,1,0)
        button.connect('clicked', ActionAddToInput,b)
        button.show()

    #########################################################        

        
    frame = gtk.Frame(" Expression ")
    frame.show()
    vbox.pack_start(frame,0,0)

    hbox = gtk.HBox()
    hbox.show()
    frame.add(hbox)

    def ActionClearInput(args):
        expression.entry.set_text("")
        return
    
    button = gtk.Button(" Clr ")
    button.connect('clicked', ActionClearInput)
    button.show()
    hbox.pack_start(button,0,0,0)


    def ActionEvalInput(args,gui):

        e = expression.entry.get_text()
        f = format.entry.get_text()
        m = mode.entry.get_text()

        try:
            tokens =  Tokenize(e, m, gui.value_history)
            result = EvaluateExpression(tokens,m)
        except SyntaxError,ex:
            gui_message("Error", ex.Message() + "  " + e,[("OK",None)])
            return

        gui.value_history.append(result) 

        iter = gui.model.append()
        gui.model.set_value(iter, 0,  (f % result) + (" [%d]" % gui.count) )
        gui.model.set_value(iter, 1,  e + " [" + f + "]"  )

        gui.count = gui.count + 1
        gui.vadj.set_value( gui.vadj.upper)
    
        # add to history

        try:
            gui.list.remove(expresion)
        except:
            x = 1

        gui.list.insert(0,e)
    
        expression.set_popdown_strings(gui.list)

        return

    expression = gtk.Combo()
    expression.set_use_arrows_always(1)
    expression.disable_activate()
    #gui.combo.set_popdown_strings(global_list)
    hbox.pack_start(expression)
    expression.entry.connect("activate",ActionEvalInput,gui)
    expression.show()
    tooltips.set_tip( expression.entry, "enter expression");


    button = gtk.Button(" Eval ")
    button.connect('clicked', ActionEvalInput,gui)
    button.show()
    hbox.pack_start(button,0,0,0)

    return vbox

#######################################################
#######################################################

def CreateWidget(tooltips):

    gui = classGui()

    vbox = gtk.VBox()
    
    gui.help = CreateWidgetHelp(gui,tooltips)
    gui.calc = CreateWidgetCalc(gui,tooltips)

    vbox.pack_start(gui.help)
    vbox.pack_start(gui.calc)

    gui.help.hide()
    vbox.show()

    return vbox

#######################################################
## 
#######################################################

def Main():

   window = gtk.Window()

   def ActionDestroy(args):
       window.destroy()
       gtk.mainquit()

   window.connect("destroy", ActionDestroy)
   window.set_title('calculator')
#   window.set_border_width(0)
#   window.set_usize(config.win.width, config.win.height)

   tooltips = gtk.Tooltips()

   widget = CreateWidget(tooltips)
   widget.set_data("tooltips",tooltips) # for gpe?
   window.add(widget)
   window.show()

   tooltips.enable()

   gtk.mainloop()

#######################################################
## 
#######################################################

if __name__ == '__main__': Main()

#######################################################
## eof
#######################################################







