#!/usr/bin/env python2
#######################################################
#
#    Winzig's Contact Manager And General Database Viewer 
#
#    Copyright (C) 2001-2003 Robert Muth <robert at muth dot org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of June 1991.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program in form of the file COPYING; 
#    if not, write to the 
#
#    Free Software Foundation, Inc. <http://www.fsf.org>
#    59 Temple Place, Suite 330, 
#    Boston, MA 02111-1307  USA
#    $Id: wcontacts.py,v 1.16 2003/12/06 20:30:42 muth Exp $
#
#######################################################

from __future__ import nested_scopes

import gtk
import sys
import os
import getopt
import re
import math
import string

import wdatabase
import wconfig
import wutil


#######################################################
#######################################################
## GUI
#######################################################
#######################################################
      
class ClassGlobal:
    def __init__(self): 
        self.editor = None
        self.browser = None
        self.browser_clist = None

        self.filter_button = None
        self.database = None
        self.id = None
        self.record_ids = None
        self.sort_tag = None
        self.browser_tags = None
        self.browser_sort_tag = None

Global = ClassGlobal()

#######################################################
#
#######################################################        

def SwitchToBrowser():
    Global.editor.get_widget().hide()
    Global.config.hide()
    Global.browser.show()
    
#######################################################
##
#######################################################

def SwitchToEditor(id):
    if id >= 0:
        Global.editor.edit_existing_record(id)
    else:
        Global.editor.edit_new_record()
    Global.config.hide()
    Global.browser.hide()
    Global.editor.get_widget().show()
    
#######################################################
##
#######################################################

def SwitchToConfig():
    Global.editor.get_widget().hide()
    Global.browser.hide()
    Global.config.show()

#######################################################
#######################################################
## CONFIG
#######################################################
#######################################################

def GetFilteredRecords():
    if Global.filter_button.get_active():
        regex_tests = []
        tags = Global.editor_tags
        for i in range (len(tags)):
            text = Global.config_entry_list[i].get_text()
            if text != "":
                regex_tests.append( (tags[i],re.compile(text,re.I)) )
            
        def test(record):
            for (tag,regex) in regex_tests:
                if not regex.search( record.get_field(tag) ): return 0
            return 1

        Global.record_ids = Global.database.get_record_ids(test)
    else:
        Global.record_ids = Global.database.get_record_ids(None)

    WidgetBrowserRedraw()
    Global.matches.set_text(str(len(Global.record_ids)))
    return

#######################################################
##
#######################################################

def CreateWidgetFilter(tooltips):

    vbox = gtk.VBox()
    vbox.show()

    hbox = gtk.HBox()
    vbox.pack_start(hbox,0,0)
    hbox.show()

    Global.config_entry_list = []

#    ok = gtk.Button("APPLY")
#    hbox.pack_start(ok,0,0)
#    ok.show()

    def ActionBackClicked(widget):
        SwitchToBrowser()
        return

    button = gtk.Button(" Back ")
    hbox.pack_start(button,0,0)
    button.show()
    button.connect("clicked",ActionBackClicked)
    tooltips.set_tip( button, " back to browser ")
    
    label = gtk.Label(" Recs:")
    hbox.pack_start(label,0,0)
    label.show()
    
    count = gtk.Entry()
    count.set_width_chars(4)
    count.set_editable(0)
    hbox.pack_start(count,0,0)
    count.show()
    tooltips.set_tip( count, " number of matches ")


    Global.matches = count

    label = gtk.Label(" Filter ")
    hbox.pack_start(label,0,0)
    label.show()
    
    def ActionFilterClicked(widget):
        GetFilteredRecords()
        return 
    
#    button = gtk.ToggleButton(" Filter ")
    button = gtk.CheckButton()
    hbox.pack_start(button,0,0)
    button.show()
    button.connect("clicked",ActionFilterClicked)
    tooltips.set_tip( button, " filter records by regex matching ")
    Global.filter_button =  button

    label = gtk.Label(" SortKey:")
    hbox.pack_start(label,0,0)
    label.show()    

    sortorder = gtk.Combo()
    sortorder.disable_activate()
    sortorder.set_popdown_strings (Global.editor_tags);
    sortorder.entry.set_max_length(3)
    sortorder.entry.set_width_chars(3)
    sortorder.entry.editable = 0

    def ActionSortingChanged(widget):
        Global.browser_sort_tag = widget.get_text()
        WidgetBrowserRedraw()
        return

    sortorder.entry.connect("changed", ActionSortingChanged)
    hbox.pack_start(sortorder,0,0)
    sortorder.show()    

    window = gtk.ScrolledWindow()
    window.set_policy(gtk.POLICY_NEVER,gtk.POLICY_ALWAYS)
    vbox.pack_start(window)
    window.show() 


    table = gtk.Table( len(Global.editor_tags), 2, 1)
    table.set_homogeneous(0)
    table.show()
    count = 0
    for tag in Global.editor_tags:

          attribute = Global.database.get_attribute(tag)
          if attribute == None:
              print "missing attribute",tag
              wutil.gui_message("ERROR","missing attribute " + tag,[("OK",None)])
              sys.exit(-1)

          text = Global.database.get_attribute(tag).name();
#          text = tag
          if attribute.required(): text += "*"
          label = gtk.Label("")
          label.set_markup( "<b>" + text + "</b>")
          label.show()

          table.attach( label,0,1,count,count+1,gtk.SHRINK,gtk.SHRINK,10,0)
            
          list = attribute.list()
          if list:
              list = [""] + list
              combo = gtk.Combo()
              entry = combo.entry
                            
              combo.set_use_arrows_always(1)
              combo.disable_activate()
              combo.set_popdown_strings(list)
              combo.show()
              table.attach( combo,1,2,count,count+1,gtk.EXPAND | gtk.FILL,gtk.SHRINK)
          else:
              entry = gtk.Entry()
              entry.show()
              table.attach( entry,1,2,count,count+1,gtk.EXPAND | gtk.FILL,gtk.SHRINK)
              
          Global.config_entry_list.append( entry )
          tooltips.set_tip( entry, attribute.name())
          count += 1
                              
    window.add_with_viewport(table)

    return vbox

#######################################################
## EDITOR WIDGET
#######################################################

class ClassEditorWidget:

    def __init__(self,tooltips):
        self._dirty = 0
        columns = [" TAG "," VALUE "]
    
        vbox = gtk.VBox()
        vbox.show()

        self._widget = vbox
        hbox = gtk.HBox()
        vbox.pack_start(hbox,0,0)
        hbox.show()

        entry_list = []

        def ActionEditorBack(widget):
            assert( not self._dirty )
            SwitchToBrowser()
            return
  
        button = gtk.Button(" Back ")
        hbox.pack_start(button,0,0) 
        button.show()
        button.connect("clicked", ActionEditorBack)
        tooltips.set_tip( button, "back to browser");
        self._cancel_button = button
        
        def ActionEditorMove(widget,dir):
            assert( not self._dirty )
            if id < 0: return
            try:
                index = dir + Global.record_ids.index( self._record_id )
                #print "new index ", index, " len ",len(gui.record_ids)
                if 0 <= index and index < len(Global.record_ids):
                    self.edit_existing_record(Global.record_ids[index])
            except:
                pass
            return
  
        button = gtk.Button(" << ")
        hbox.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionEditorMove,-1)
        tooltips.set_tip( button, "previous record");
        self._left = button
        
        button = gtk.Button(" >> ")
        hbox.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionEditorMove,+1)
        tooltips.set_tip( button, "next record");
        self._right = button

        def ActionEditorReset(widget):
            self.redraw()
            self.set_dirty(0)
            return
        
        button = gtk.Button(" Reset ")
        hbox.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionEditorReset)
        tooltips.set_tip( button, "reset changes");
        self._reset = button
    
        def ActionEditorSave(widget):
            assert( self._record_id < 0 )
            assert( self._dirty)
            record = self.extract_record()
        
            try:
                self._record_id = Global.database.add_record(record)
                Global.database.write_to_file()
            except wdatabase.DatabaseError,e:
                wutil.gui_message("ERROR",e.message(),[("OK",None)])
            else:
                self.set_dirty(0)
                GetFilteredRecords()
            return                      

        button = gtk.Button(" Save ")
        hbox.pack_start(button,0,0) 
        button.show()
        button.connect("clicked", ActionEditorSave)
        tooltips.set_tip( button, "save new record")
        self._save = button
        
        def ActionEditorUpdate(widget):
            assert( self._record_id >= 0 )
            assert( self._dirty)
            record = self.extract_record()
            
            try:
                Global.database.update_record(self._record_id,record)
                Global.database.write_to_file()
            except database.DatabaseError,e:
                wutil.gui_message("ERROR",e.message(),[("OK",None)])
            else:
                self.set_dirty(0)
                GetFilteredRecords()
            return                      

        button = gtk.Button(" Update ")
        hbox.pack_start(button,0,0) 
        button.show()
        button.connect("clicked", ActionEditorUpdate)
        tooltips.set_tip( button, "update existing record")
        self._update = button
    
        def ActionEditorDel(widget):
            assert( self._record_id >= 0)

            def DelRecord():
                try:
                    Global.database.del_record(self._record_id)
                    Global.database.write_to_file()
                except database.DatabaseError,e:
                    wutil.gui_message("ERROR",e.message(),[("OK",None)])
                else:
                    self._record_id = -1
                    tags = Global.editor_tags
                    for i in range (len(tags)):
                        self._entry_list[i].set_text("")
                self.set_dirty(0)                                
                GetFilteredRecords()
                return                      

            wutil.gui_message("del","Delete Record",[("OK",DelRecord),("CANCEL",None)])
            return

        button = gtk.Button(" Del ")
        hbox.pack_end(button,0,0)
        button.show()
        button.connect("clicked", ActionEditorDel)
        tooltips.set_tip( button, "remove record ")
        self._delete = button

        window = gtk.ScrolledWindow()
        window.set_policy(gtk.POLICY_NEVER,gtk.POLICY_AUTOMATIC)
        vbox.pack_start(window)
        window.show() 

        def ActionChanged(widget):
            self.set_dirty(1)
            return

        table = gtk.Table( len(Global.editor_tags), 2, 0)
#        table.set_col_spacing(0,0)
#        table.set_col_spacing(1,0)
        table.set_homogeneous(0)
        table.show()
        count = 0

        self._entry_list = []
        for tag in Global.editor_tags:

            attribute = Global.database.get_attribute(tag)
            if attribute == None:
                print "missing attribute",tag
                wutil.gui_message("ERROR","missing attribute " + tag,[("OK",None)])
                sys.exit(-1)

            text = Global.database.get_attribute(tag).name();
            #text = tag

            if attribute.required(): text += "*"
            label = gtk.Label("")
            label.set_markup( "<b>" + text + "</b>")
            label.show()
            table.attach( label,0,1,count,count+1,gtk.SHRINK,gtk.SHRINK,10,0)
            
            list = attribute.list()
            if list:
                combo = gtk.Combo()
                entry = combo.entry
                entry.set_editable(0) 
                
                combo.set_use_arrows_always(1)
                combo.disable_activate()
                combo.set_popdown_strings(list)
                combo.show()
                table.attach( combo,1,2,count,count+1,gtk.EXPAND | gtk.FILL,gtk.SHRINK)
            else:
                entry = gtk.Entry()
                entry.show()
                table.attach( entry,1,2,count,count+1,gtk.EXPAND | gtk.FILL,gtk.SHRINK)

            self._entry_list.append( entry )
            entry.connect("changed",ActionChanged)
            tooltips.set_tip( entry, attribute.name())
            count += 1
            
        window.add_with_viewport(table)

    def set_dirty(self,dirty):
        if self._dirty == dirty: return
        self._dirty = dirty

        if dirty:
            self._cancel_button.hide()
            self._reset.show()

            if self._record_id >= 0:
                self._update.show()
                self._left.hide()
                self._right.hide()
            else:
                self._save.show()
        else:
            self._cancel_button.show()
            self._reset.hide()
            self._update.hide()
            self._save.hide()
            
            if self._record_id >= 0:
                self._delete.show()
                self._left.show()
                self._right.show()
                
            else:
                self._delete.hide()
                self._left.hide()
                self._right.hide()
        return

    def get_widget(self):
        return self._widget

    def extract_record(self):
        # we need to preserve the original record for reset purposes
        record = wdatabase.ClassRecord()

        for i in range (len(Global.editor_tags)):
            record.update_field(Global.editor_tags[i],self._entry_list[i].get_text())
        return record
        

    def redraw(self):
        for i in range (len(Global.editor_tags)):
            self._entry_list[i].set_text(self._record.get_field(Global.editor_tags[i]))
        return
        
    def edit_existing_record(self,id):
        self._record_id = id
        self._record = Global.database.get_record(id)
        self.redraw()
        self.set_dirty(0)
        return

    def edit_new_record(self):
        self._record_id = -1
        self._record = wdatabase.ClassRecord()
        self.redraw()
        self.set_dirty(0)
        return

#######################################################
## BROWSER REDRAW
#######################################################

def WidgetBrowserRedraw():

    tag = Global.browser_sort_tag
    
    def cmp(r1,r2):
        s1 = string.lower(Global.database.get_record(r1).get_field(tag))
        s2 = string.lower(Global.database.get_record(r2).get_field(tag))
        if s1 < s2: return -1
        elif s1 == s2: return 0 
        else: return +1
            
    Global.record_ids.sort( cmp )
    tags = Global.browser_tags
    clist = Global.browser_clist
    
    clist.freeze()
    clist.clear()

    count = 0
    for id in Global.record_ids:
        record = Global.database.get_record(id)
        
        row = map( lambda x: record.get_field(x),tags)
        clist.append( row )
        count = count + 1
    clist.thaw()
    return

#######################################################
## BROWSER CREATION
#######################################################
GlobalLastStart = '#'

def CreateWidgetBrowser(tooltips):

    titles =  map( lambda x: Global.database.get_attribute(x).name(),Global.browser_tags)
    cols = len(titles)

    clist = gtk.CList( cols, titles )

    Global.browser_clist = clist
    
 #   font = clist.get_style().get_font()
 #   normal_width = 2*font.char_width("M")

    for col in range( cols ):
        clist.set_column_auto_resize(col, True) 
#        clist.set_column_width(col,normal_width)
      
    vbox = gtk.VBox()
    vbox.show()

    #######################################################
    
    hbox = gtk.HBox()
    vbox.pack_start(hbox,0,0)
    hbox.show()

    def ActionWidgetBrowserTabClicked(widget,key):
        global GlobalLastStart
        if GlobalLastStart == key[0]: GlobalLastStart = key[1]
        elif GlobalLastStart == key[1]: GlobalLastStart = key[2]
        else: GlobalLastStart = key[0]
        
        tag = Global.browser_sort_tag
        count = 0
        for id in Global.record_ids:
            record = Global.database.get_record(id)
            value = string.upper(record.get_field(tag))
            if value[0] >= GlobalLastStart:
                clist.moveto(count,0,0,0)
                return
            count = count + 1       
        return

    list  = ["#ab","cde","fgh","ijk","lmn","opq","rst","uvw","xyz"]

    for l in list:
        button = gtk.Button(l)
        hbox.pack_start(button,1,1)
        button.show()
        button.connect("clicked",ActionWidgetBrowserTabClicked,string.upper(l))

    #######################################################
    
    window = gtk.ScrolledWindow()
    window.set_policy(gtk.POLICY_ALWAYS,gtk.POLICY_ALWAYS)
    vbox.pack_start(window)
    window.show()

            
    clist.show()
    window.add(clist)

    def ActionWidgetBrowserTitleClicked(widget,col):
        SwitchToEditor()
        return

        # the  gtk2.0 does no support get_column-width  
        if len(Global.record_ids) == 0:
            SwitchToEditor()
            return
        
        tag = Global.browser_tags[col]
        width = clist.get_column_width(col)
        if width != normal_width:
            width = normal_width
            title = tag
        else:
            width = clist.optimal_column_width(col)
            title = Global.database.get_attribute(tag).name()
        clist.set_column_width(col,width)
        clist.set_column_title(col,title)
        return


#    clist.connect("click-column", ActionWidgetBrowserTitleClicked)

    def ActionWidgetBrowserRowClicked(widget,row,col, event):
        SwitchToEditor(Global.record_ids[row])
        return

    clist.connect("select-row", ActionWidgetBrowserRowClicked)

    #######################################################
    
    hbox = gtk.HBox()
    vbox.pack_start(hbox,0,0)
    hbox.show()

    def ActionWidgetNewClicked(widget):
        SwitchToEditor(-1)
        return


    button = gtk.Button("New")
    hbox.pack_start(button,0,0)
    button.show()
    button.connect("clicked",ActionWidgetNewClicked)

          
    def ActionWidgetBrowserConfigClicked(widget):
        SwitchToConfig()
        return

    button = gtk.Button("Find")
    hbox.pack_end(button,0,0)
    button.show()
    button.connect("clicked",ActionWidgetBrowserConfigClicked)
    tooltips.set_tip( button, " find records using regular expression ")
          

    
    return vbox

#######################################################
## TOPLEVEL CREATE
#######################################################

def CreateWidget(tooltips, filename=None):

    Global.database = wdatabase.ClassDatabase()

    if not filename:
        filename = wconfig.contacts.database

    try:
        Global.database.read_from_file(filename)
    except wdatabase.DatabaseError,e:
        print e.message()
         
    Global.browser_tags = Global.database.get_attribute("sb").list()
    Global.editor_tags = Global.database.get_attribute("se").list()

    Global.browser_sort_tag = Global.database.get_attribute("sk").value()

    vbox = gtk.VBox()
    
    Global.browser = CreateWidgetBrowser(tooltips)

    Global.editor = ClassEditorWidget(tooltips)
    Global.config = CreateWidgetFilter(tooltips)

    vbox.pack_start(Global.browser)
    vbox.pack_start(Global.editor.get_widget())
    vbox.pack_start(Global.config)

    GetFilteredRecords()
    WidgetBrowserRedraw()
#   WidgetEditorSetDirty(gui,0)
    SwitchToBrowser()

    vbox.show()

    return vbox

#######################################################
## 
#######################################################

def Main():

    window = gtk.Window()

    def ActionDestroy(args,window):
        window.destroy()
        gtk.mainquit()

    window.connect("destroy", ActionDestroy,window)
    window.set_title('Contacts')
    window.set_border_width(0)
#    window.set_usize(wconfig.win.width, wconfig.win.height)

    tooltips = gtk.Tooltips()

    if len(sys.argv) > 1:
        widget = CreateWidget(tooltips,sys.argv[1] )
    else:
        widget = CreateWidget(tooltips )
        
    window.add(widget)
    window.show()

    tooltips.enable()

    gtk.mainloop()

#######################################################
## 
#######################################################

if __name__ == '__main__':
    if 1:
          Main()
    else:
          import profile
          import pstats

          profile.run("Mmain","run.prof")    
          p = pstats.Stats("run.prof")
          p.sort_stats('cumulative').print_stats(20)
          p.sort_stats('time').print_stats(20)

#######################################################
## eof
#######################################################

