#!/usr/bin/env python2
#######################################################
#
#    Winzig's Todo List Manager
#
#    Copyright (C) 2001-2003 Robert Muth <robert at muth dot org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of June 1991.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program in form of the file COPYING; 
#    if not, write to the 
#
#    Free Software Foundation, Inc. <http://www.fsf.org>
#    59 Temple Place, Suite 330, 
#    Boston, MA 02111-1307  USA
#    $Id: wtodo.py,v 1.13 2003/11/18 22:38:40 muth Exp $
#
#######################################################

from __future__ import nested_scopes


import time 
import re 
import string
import os

import gtk
import gobject

import wconfig
import wdatabase
from wutil import *
#######################################################
#
#######################################################        

def get_current_time_string():
    return time.strftime("%Y/%m/%d %H:%M",time.localtime(time.time()))

#######################################################
##
#######################################################

class classGui:
    def __init__(self): 
        self.edit = None
        self.view  = None
        self.editcategory    = None
        self.editpriority    = None
        self.editstate       = None
        self.editfulltext    = None
        self.editdescription = None
        self.viewcategory    = None
        self.viewcstate      = None
        self.model = None

        self.database = None
        self.id = None

#######################################################
##
#######################################################

def RedrawView(gui):
    def filter(record):
        if gui.viewcategory != "all":
            if record.get_field("c1") != gui.viewcategory:
                return 0
        if gui.viewstate != "all":
            if record.get_field("s1") != gui.viewstate:
                return 0
        return 1
    
    id_list =  gui.database.get_record_ids( filter )

    # FIXME hack in sorting
    
    gui.model.clear()
    for id in id_list:
        record = gui.database.get_record(id)

        iter = gui.model.append()
        state =  record.get_field("s1")
        gui.model.set_value(iter, 0, id)
        priority =  record.get_field("p1")
        if state == "closed": priority = priority + "*"
        gui.model.set_value(iter, 1, priority )
        gui.model.set_value(iter, 2, record.get_field("c1") )
        gui.model.set_value(iter, 3 ,record.get_field("d1") ) 
    
#######################################################
#######################################################

def SwitchToEdit(gui,id):
    if id >= 0:
        record = gui.database.get_record( id )
    else:
        record = gui.empty_record

#    print "switch",id,record._fields
    
    gui.editcategory.set_text( record.get_field("c1"))
    gui.editpriority.set_text( record.get_field("p1"))
    gui.editstate.set_text( record.get_field("s1"))
    gui.editdescription.set_text( record.get_field("d1"))

    gui.id = id

    gui.view.hide()
    gui.edit.show()

#######################################################
##
#######################################################

def SwitchToView(gui):
    RedrawView(gui)
    gui.edit.hide()
    gui.view.show()
    return

#######################################################
## 
#######################################################

def SaveDbRecord(database, id, record):
    try:
        if id >=0:
            #print "updating existing record"
            database.update_record(id,record)
            database.write_to_file()
        else:
            #print "creating new record"
            database.add_record(record,0)

            database.write_to_file()
        return 1    

    except wdatabase.DatabaseError,e:
        gui_message("ERROR",e.message(),[("OK",None)])
        return 0


#######################################################
##
#######################################################

def CreateWidgetEdit(gui,tooltips):

    list_categories = gui.database.get_attribute("c1").list()
    list_priorities = gui.database.get_attribute("p1").list()
    list_states = gui.database.get_attribute("s1").list()

    gui.empty_record = wdatabase.ClassRecord()
    gui.empty_record.update_field("c1",list_categories[0])
    gui.empty_record.update_field("p1",list_priorities[0])
    gui.empty_record.update_field("s1",list_states[0])
    
    vbox = gtk.VBox()
    vbox.show()

    hbox = gtk.HBox()
    vbox.pack_start(hbox,0,0)
    hbox.show()

    def ActionBack(args,gui):
        SwitchToView(gui)
        return

    button = gtk.Button(" Cancel ")
    hbox.pack_start(button,0,0)
    button.connect('clicked', ActionBack,gui)
    button.show()
    tooltips.set_tip( button, "cancel");
    
    combo = gtk.Combo()
    combo.set_use_arrows_always(1)
    combo.disable_activate()
    combo.set_popdown_strings(list_categories)
    combo.show()
    hbox.pack_start(combo,0,0)
    combo.entry.set_width_chars(10)
    combo.entry.set_editable(0)
    tooltips.set_tip( combo.entry, "select category");

    gui.editcategory = combo.entry
    
    combo = gtk.Combo()
    combo.set_use_arrows_always(1)
    combo.disable_activate()
    combo.set_popdown_strings(list_priorities)
    combo.show()
    hbox.pack_start(combo,0,0)
    combo.entry.set_width_chars(3)
    combo.entry.set_editable(0)
    tooltips.set_tip( combo.entry, "select priority");
 
    gui.editpriority = combo.entry
   
    gui.combo = gtk.Combo()
    
    combo = gtk.Combo()
    combo.set_use_arrows_always(1)
    combo.disable_activate()
    combo.set_popdown_strings(list_states)
    hbox.pack_start(combo,0,0)
    combo.show()
    combo.entry.set_editable(0)
    combo.entry.set_width_chars(10)
    tooltips.set_tip( combo.entry, "select state");

    gui.editstate = combo.entry
    
    def ActionSaveRecord(args,gui):
        record = wdatabase.ClassRecord()
        record.update_field("c1",gui.editcategory.get_text())
        record.update_field("p1",gui.editpriority.get_text())
        record.update_field("s1",gui.editstate.get_text())
        record.update_field("d1",gui.editdescription.get_text())

        if SaveDbRecord(gui.database,gui.id,record):
            SwitchToView(gui)

        return

    button = gtk.Button(" Save ")
    hbox.pack_end(button,0,0)
    button.connect('clicked', ActionSaveRecord,gui)
    button.show()
    tooltips.set_tip( button, "update/save record");
        
    entry = gtk.Entry()
    vbox.pack_start( entry,0,0)
    entry.set_editable(1)
    entry.show()
    tooltips.set_tip( entry, "short descritption");

    gui.editdescription = entry
    
#    gui.editfulltext = gtk.Text()
#    gui.editfulltext.set_editable(1)
#    vbox.pack_start(gui.editfulltext,1,1)
#    gui.editfulltext.show()
#    tooltips.set_tip( gui.editfulltext, "long descritption");
    
    return vbox

#######################################################
##
#######################################################
        
def CreateWidgetView(gui,tooltips):

    vbox = gtk.VBox()
    vbox.show()

    hbox = gtk.HBox()
    vbox.pack_start(hbox,0,0)
    hbox.show()

    #######################################################
    
    
    label = gtk.Label(" Category ")
    hbox.pack_start(label,0,0)
    label.show()


    list_categories = ["all"]
    list_categories += gui.database.get_attribute("c1").list()

    gui.viewcategory = list_categories[0]
    
    combo = gtk.Combo()
    combo.set_use_arrows_always(1)
    combo.disable_activate()
    combo.set_popdown_strings( list_categories)
    hbox.pack_start(combo,0,0)

    def ActionViewCategoryChanged(entry,gui):
        gui.viewcategory = entry.get_text()
        #print  gui.viewcategory
        RedrawView(gui)
        return

    combo.show()
    combo.entry.set_editable(0)
    combo.entry.set_width_chars(8)
    combo.entry.connect("changed",ActionViewCategoryChanged,gui)
    tooltips.set_tip( combo.entry, "select category filter"); # work around a bug?

    #######################################################
    
    label = gtk.Label(" State ")
    hbox.pack_start(label,0,0)
    label.show()

    list_categories = gui.database.get_attribute("s1").list()
    list_categories += ["all"]

    gui.viewstate = list_categories[0]
    
    combo = gtk.Combo()
    combo.set_use_arrows_always(1)
    combo.disable_activate()
    combo.set_popdown_strings( list_categories)
    hbox.pack_start(combo,0,0)

    def ActionViewStateChanged(entry,gui):
        gui.viewstate = entry.get_text()
        #print gui.viewstate
        RedrawView(gui)
        return

    combo.show()
    combo.entry.set_editable(0)
    combo.entry.set_width_chars(8)
    combo.entry.connect("changed",ActionViewStateChanged,gui)
    tooltips.set_tip( combo.entry, "select state filter");

    #######################################################
    
#    button = gtk.ToggleButton(" Del ")
#    button.connect('toggled', ActionDelete)
#    hbox.pack_end(button,0,0)
#    button.show()
#    gui.tooltips.set_tip( button,"Delete Item");
#    gui.active = button


    window = gtk.ScrolledWindow()
    window.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
    window.set_shadow_type (gtk.SHADOW_ETCHED_IN);
    vbox.pack_start(window)
    window.show() 


    model = gtk.ListStore(gobject.TYPE_PYOBJECT, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)

    gui.model = model

    view = gtk.TreeView(model)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("Pri", renderer, text=1)
    column.set_sort_column_id(1)
    column.set_resizable(1)
    view.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("Cat", renderer, text=2)
    column.set_sort_column_id(2)
    column.set_resizable(1)
    view.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("Description", renderer, text=3)
    column.set_sort_column_id(3)
    column.set_resizable(1)
    view.append_column(column)

    def ActionEditItem(arg1,path,col,gui):
        iter = gui.model.get_iter(path)
        id = gui.model.get_value(iter,0)
        #print   col.get_sort_column_id()
        SwitchToEdit(gui,id)
        return


    view.connect("row_activated",ActionEditItem,gui)
    view.set_search_column(1);

    view.show()
    window.add_with_viewport(view)

    #######################################################
    
    hbox = gtk.HBox()
    vbox.pack_start(hbox,0,0)
    hbox.show()

    #######################################################

    label = gtk.Label(" Item ")
    hbox.pack_start(label,0,0)
    label.show()
    
    def GetCurrentDbId():
        selection = view.get_selection()
        (dummy,iter) =  selection.get_selected()
        if not iter: return -1
        return model.get_value(iter,0)


        
    def ActionNewItem(args,gui):
        SwitchToEdit(gui,-1)
        return

    button = gtk.Button(" Add ")
    hbox.pack_start(button,0,0)
    button.connect('clicked', ActionNewItem,gui)
    button.show()
    tooltips.set_tip( button,"Add Item");

    def ActionCloseItem(args,gui):
        id = GetCurrentDbId()
        if id < 0: return
        record = gui.database.get_record( id )
        record.update_field("s1","closed")
        SaveDbRecord(gui.database,id,record)
        RedrawView(gui)
        return
    
    button = gtk.Button(" Close ")
    hbox.pack_start(button,0,0)
    button.connect('clicked', ActionCloseItem,gui)
    button.show()
    tooltips.set_tip( button,"Close Selected Item");

    def ActionDeleteItem(args,gui):
        id = GetCurrentDbId()
        assert( id >= 0 )
        gui.database.del_record(id)
        gui.database.write_to_file()

        RedrawView(gui)
        return
    
    button = gtk.Button(" Delete ")
    hbox.pack_end(button,0,0)
    button.connect('clicked', ActionDeleteItem,gui)
    button.show()
    tooltips.set_tip( button,"Delete Selected Item Permanently");

    return vbox

#######################################################
##
#######################################################

def CreateWidget(tooltips):
    gui = classGui()

#    database.load(config.todo.database)
#    UpdateList()


    gui.database = wdatabase.ClassDatabase()
    gui.database.read_from_file(wconfig.todo.database)

    vbox = gtk.VBox()

    gui.edit = CreateWidgetEdit(gui,tooltips)
    gui.view = CreateWidgetView(gui,tooltips)

    vbox.pack_start(gui.edit)
    vbox.pack_start(gui.view)

    SwitchToView(gui)
#    RedrawView(gui)
    
    vbox.show()

    return vbox

#######################################################
## 
#######################################################


if __name__ == '__main__':

    window = gtk.Window()

    def ActionDestroy(args):
        window.destroy()
        gtk.mainquit()

    window.connect("destroy", ActionDestroy)
    window.set_title('TODO')
#    window.set_border_width(0)
#    window.set_usize(wconfig.win.width, wconfig.win.height)

    tooltips = gtk.Tooltips()
    
    widget = CreateWidget(tooltips)

    tooltips.enable()


    window.add(widget)
    window.show()
    gtk.mainloop()

#######################################################
## eof
#######################################################







