#!/usr/bin/env python2
#######################################################
#
#    Zettel: Winzig's notebook
#
#    Copyright (C) 2001-2003 Robert Muth <robert at muth dot org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of June 1991.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program in form of the file COPYING; 
#    if not, write to the 
#
#    Free Software Foundation, Inc. <http://www.fsf.org>
#    59 Temple Place, Suite 330, 
#    Boston, MA 02111-1307  USA
#    $Id: wzettel.py,v 1.10 2003/12/04 22:37:20 muth Exp $
#
#######################################################

from __future__ import nested_scopes


import time 
import re 
import string
import os
import sys

import gtk
import gobject
import pango

import wconfig
import wdatabase
import wutil 

#######################################################
#
#######################################################        

def get_current_time_string():
    return time.strftime("%Y/%m/%d %H:%M",time.localtime(time.time()))

#######################################################

def ReadTaggedBufferFromFile(buffer, filename):
    iter =  buffer.get_start_iter()
    buffer.delete( iter,  buffer.get_end_iter() )

    try:
        input = open(filename, "r")
        fulltext = unicode(input.read(),'utf-8')
        input.close()
    except:
        #wutil.Info("could not open file " + filename)
        fulltext = ""

    tags = []
    start = 0
    while 1:
        end = string.find(fulltext,u"@[",start)
        if end < 0: data = fulltext[start:]
        else: data = fulltext[start:end]
        
        #buffer.insert_with_tags_by_name(iter, data.encode("utf-8"), tags)
        # gross but what do you do if the API is just not right
        apply( buffer.insert_with_tags_by_name, [iter, data.encode("utf-8")] + tags)
        if end < 0: break
        
        start = end + 2
        end = string.find(fulltext,u"]",start)
        assert( end > 0 )
        for t in  string.split(fulltext[start:end]):
            flag = t[0]
            name = t[1:].encode("ascii")
            if flag == "+":
                assert( name not in tags )
                tags.append(name)
            elif flag == '-':
                assert( name in tags )
                tags =  filter( lambda x: x != name, tags)
            else:
                assert(0)

        start = end + 1
        
            
#######################################################
def current_tag_string(iter):
    tags_off = map( lambda x: "-" + x.get_property("name") , iter.get_toggled_tags(0))
    tags_on  = map( lambda x: "+" + x.get_property("name"),  iter.get_toggled_tags(1))
    tags =  tags_on + tags_off
    if tags: return "@[" + string.join(tags) + "]"        
    else: return ""

#######################################################
    
def WriteTaggedBufferToFile(buffer, filename):
    fulltext = ""

    start =  buffer.get_start_iter()
    fulltext += current_tag_string(start)

    while not start.is_end():
        end = start.copy()
        end.forward_to_tag_toggle(None)

        data = buffer.get_text(start,end,1)
        fulltext += data
        fulltext += current_tag_string(end)
        start = end

    #wutil.Info("writing file " + filename)    
    try:
        output = open(filename, "w+")
        output.write(fulltext)
        output.close()
    except:
        wutil.gui_message("Warning","could not write file " + filename,[("OK",None)])
    return


#######################################################
##
#######################################################

class ClassGlobal:
    def __init__(self): 
        self._edit = None
        self._tree_view  = None
        self._database = None

Global = ClassGlobal()

#######################################################
##
#######################################################

def SwitchToView():
    global Global
    Global._edit.get_widget().hide()
    Global._tree_view.draw_all_zettel( ClassZettel.Zettels.values() )
    Global._tree_view.get_widget().show()
    return

#######################################################
class ClassZettel:

    Zettels = {} # holds all class instances
    RootTag = ""

    def __init__(self):
        return

    def init(self,t,n):
        self._parent = None
        self._tag = t
        self._name = n
        self._children = []
        self._expanded = 0

        ClassZettel.Zettels[t] = self
        return

    def link_to_parent(self, parent,sibling=None):
        """link a child to its parent after a given sibling""" 
        assert( not self._parent)
        self._parent = parent
        if sibling:
            parent._children.insert(parent._children.index(sibling),self)
        else:
            parent._children.append(self)
        return

    def unlink_from_parent(self):
        """unlink a child from its parent""" 
        assert( self._parent)
        self._parent._children.remove(self)
        self._parent = None
        return

    def delete(self):
        """remove zettel"""

        parent = self._parent
        for child in self._children:
            child.unlink_from_parent()
            child.link_to_parent(parent)


        assert( len(self._children) == 0 )
        
        self.unlink_from_parent()
        del ClassZettel.Zettels[self._tag]
        return parent

#######################################################
##    
#######################################################    

def FindUnusedTag(stem = "new" ):
    i = 0
    while 1:
        tag = stem + ("%d" % i)
        if not ClassZettel.Zettels.has_key(tag): return tag
        i += 1
    return

#######################################################
##
#######################################################

def FindZettelByTag(tag):
    return ClassZettel.Zettels[tag]

#######################################################
##
#######################################################

def FindZettelRoot():
    return FindZettelByTag(ClassZettel.RootTag)

#######################################################
##
#######################################################

def ReadDatabase(database,filename):

    try:
#        print filename
        database.read_from_file(filename)
    except wdatabase.DatabaseError,e:
        wutil.gui_message("Error",   "error reading db " + filename,[("OK",None)])
        print e.message()


    ids = database.get_record_ids(None)
    #print ids
    
    root = ClassZettel()
    root.init(ClassZettel.RootTag,"Top Level")
    # root is only unlinked node
    
    for id in ids:
        rawrec = database.get_record(id)
        zettel = ClassZettel()
        ptag = rawrec.get_field("p1")
        parent = FindZettelByTag(ptag)
        
        tag = rawrec.get_field("t1")
        name = rawrec.get_field("n1")
        zettel.init(tag,name)

        zettel.link_to_parent(parent)

    return    

#######################################################
##
#######################################################

def AddChildrenToDbRecursively(database,parent):
    for child in parent._children:
        rawrec = wdatabase.ClassRecord()
        rawrec.update_field("p1",child._parent._tag)
        rawrec.update_field("t1",child._tag)
        rawrec.update_field("n1",child._name)

        database.add_record(rawrec)
        AddChildrenToDbRecursively(database,child)
    return    

#######################################################
##
#######################################################

def WriteDatabase(database):

    # clear database
    ids = database.get_record_ids(None)
    for id in ids:
        database.del_record(id)

    root = FindZettelRoot()
    AddChildrenToDbRecursively(database,root)
    database.write_to_file(0)   
    return

#######################################################
##
#######################################################

class ClassTreeView:

    def __init__(self, tooltips):
        self._widget = None
        self._model = None
        self._view = None
        self._connect_id = None

        #######################################################

        vbox = gtk.VBox()

        #######################################################
        def ActionExpand(widget,iter,path,flag):
            global Global
            tag = self._model.get_value(iter, 1)
            record = FindZettelByTag(tag)
            record._expanded = flag
#            print "collapse ",tag,path,flag            
            return

        #######################################################
        
        def ActionEditItem(widget,path,col):
            iter = self._model.get_iter(path)
            record = self._model.get_value(iter, 0)
            SwitchToEditor(record)
            return



        

        window = gtk.ScrolledWindow()
        window.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
        window.show() 

#        model = gtk.TreeStore(gobject.TYPE_PYOBJECT, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING,gobject.TYPE_STRING)
        model = gtk.TreeStore(gobject.TYPE_PYOBJECT, gobject.TYPE_STRING, gobject.TYPE_STRING)

        self._model = model

        view = gtk.TreeView(model)

        self._view = view
        
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn("Tag", renderer, text=1)
        #column.set_sort_column_id(1)
        column.set_resizable(1)
        view.append_column(column)

        column = gtk.TreeViewColumn("Title", renderer, text=2)
        #column.set_sort_column_id(1)
        column.set_resizable(1)
        view.append_column(column)

#        renderer = gtk.CellRendererText()
#        column = gtk.TreeViewColumn("Created", renderer, text=3)
        #column.set_sort_column_id(2)
#        column.set_resizable(1)
#        view.append_column(column)

#        renderer = gtk.CellRendererText()
#        column = gtk.TreeViewColumn("Modified", renderer, text=4)
        #column.set_sort_column_id(2)
#        column.set_resizable(1)
#        view.append_column(column)

        view.connect("row-expanded",ActionExpand,1)

        view.connect("row-collapsed",ActionExpand,0)

        view.connect("row_activated",ActionEditItem)
        
        view.set_enable_search(1)
        view.set_search_column(1)
        
        view.show()
        window.add_with_viewport(view)
    
        vbox.pack_start(window,1,1)

        
        #######################################################
        # Buttons
        #######################################################

        hbox = gtk.HBox()
        vbox.pack_start(hbox,0,0)
        hbox.show()

        #######################################################
        
        def ActionNew(widget):
            global Global
            selection = self._view.get_selection()
            (dummy,iter) =  selection.get_selected()
            if iter:
                parent =  self._model.get_value(iter, 0)
            else:
                parent = FindZettelRoot()


            zettel = ClassZettel()
            zettel.init( FindUnusedTag(),"")
            zettel.link_to_parent(parent)

            WriteDatabase(Global._database)
            self.draw_all_zettel(FindZettelRoot)
            return
            
        button = gtk.Button(" New ")
        hbox.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionNew)    

        #######################################################
        
        def ActionDup(widget):
            global Global
            selection = self._view.get_selection()
            (dummy,iter) =  selection.get_selected()
            if not iter: return
                
            template = self._model.get_value(iter, 0)
            filename =  Global._path + template._tag
            
            zettel = ClassZettel()
            zettel.init( FindUnusedTag(template._tag),template._name)
            zettel.link_to_parent(template._parent,template)

            if os.path.exists(filename):
                input = open(filename, "r")
                fulltext = input.read()
                input.close()

                filename = Global._path + zettel._tag
                output = open(filename, "w+")
                output.write(fulltext)
                output.close()

            WriteDatabase(Global._database)
            self.draw_all_zettel(FindZettelRoot)
            return
            
        button = gtk.Button(" Dup ")
        hbox.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionDup)    

        #######################################################
        
        def ActionDel(widget):
            global Global
            selection = self._view.get_selection()
            (dummy,iter) =  selection.get_selected()
            if not iter: return

            zettel = self._model.get_value(iter, 0)
            filename =  Global._path + zettel._tag
            
            if os.path.exists(filename):
                def DelRecord():
                    zettel.delete()
                    os.remove(filename)
                    WriteDatabase(Global._database)
                    self.draw_all_zettel(FindZettelRoot)
                    return                      

                wutil.gui_message("del","Delete Entry",[("OK",DelRecord),("CANCEL",None)])
            else:
                zettel.delete()
                WriteDatabase(Global._database)
                self.draw_all_zettel(FindZettelRoot)
            return

        button = gtk.Button(" Del ")
        hbox.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionDel)    

        #######################################################
        
        def ActionEnableReorder(widget):
            view.set_reorderable(widget.get_active())                
            return
            
        button = gtk.CheckButton(" Reorder ")
        hbox.pack_end(button,0,0)
        button.show()
        button.connect("toggled",ActionEnableReorder)
        
        #######################################################
        
#        label = gtk.Label("XXXXXXXX")
#        label.show()
#        hbox.pack_end(label,0,0)
#        self._dirty_label = label

        #######################################################
        vbox.show()
        self._widget = vbox
        return

    def draw_zettel_recursive(self, record, parent_iter):
        iter = self._model.append(parent_iter)
        self._model.set_value(iter, 0, record )
        self._model.set_value(iter, 1, record._tag )
        self._model.set_value(iter, 2, record._name )

        for child in record._children:
            self.draw_zettel_recursive(child,iter)

        # now that children are draw expand if necessary
        if record._expanded: 
            path =  self._model.get_path(iter)
            self._view.expand_row(path,0)
        return
        
    def draw_all_zettel(self,zettels):

        def ActionReordered(widget, path, iter):
            global Global
            me = self._model.get_value(iter, 0)

            parent = self._model.iter_parent( iter)
            if parent == None:
                parent = FindZettelRoot()
            else:
                parent =  self._model.get_value(parent, 0)

            sibling = self._model.iter_next( iter)
            if sibling != None:
                sibling =  self._model.get_value(sibling, 0)

            me.unlink_from_parent()
            me.link_to_parent(parent,sibling)
            WriteDatabase(Global._database)
#            print "reordered ",me._tag,parent._tag
            return

        if self._connect_id:
            self._model.disconnect(self._connect_id)

            
        self._model.clear()
        for x in  FindZettelRoot()._children:
            self.draw_zettel_recursive(x,None)


#        self._connect_id = self._model.connect("rows-reordered",ActionReordered)
        self._connect_id = self._model.connect("row-changed",ActionReordered)
        return
    
    def get_widget(self):
        return self._widget


#######################################################
## EDITOR STUFF
#######################################################

def SwitchToEditor(record):
    global Global
    Global._tree_view.get_widget().hide()
    Global._edit.set_edit_record(record)
    Global._edit.get_widget().show()
    return

#######################################################

class ClassEditZettel:

    def set_dirty(self, dirty):
        if self._dirty == dirty: return
        self._dirty = dirty
        if dirty:
            self._update_button.show()
            self._reset_button.show()
        else: 
            self._update_button.hide()
            self._reset_button.hide()
        return
    
    def __init__(self, tooltips):
        global Global
        self._widget = None
        self._dirty = 0
        self._last_change = None

        self._id = None

        # record widgets:
        
        self._tag = None
        self._parents = None
        self._last_change = None
        self._content = None
        self._title = None 
        self._children = []
        self._filepath = ""
        
        #######################################################
        def ActionSetDirty(widget):
            self.set_dirty(1)
            return
        #######################################################

        vbox = gtk.VBox()

        #######################################################
        hbox = gtk.HBox()
        hbox.show()
        vbox.pack_start(hbox,0,0)
        
        #######################################################
        frame = gtk.Frame(" Tag ")
        frame.show()
        hbox.pack_start(frame,0,0)

        entry = gtk.Entry()
        entry.show()
        entry.connect("changed",ActionSetDirty)
        frame.add(entry)
        
        self._tag = entry

        #######################################################

        frame = gtk.Frame(" Title ")
        frame.show()
        hbox.pack_start(frame,1,1)
        
        entry = gtk.Entry()
        entry.show()
        entry.connect("changed",ActionSetDirty)
        frame.add(entry)

        self._title = entry

        #######################################################

        def ActionGoTo(widget):
            global Global
            if  self._dirty:
                 wutil.gui_message("Warning","Reset or Save Changes",[("OK",None)])
            else:    
                self.set_edit_record( FindZettelByTag(widget.get_label()))
            return

        #######################################################
        hbox = gtk.HBox()
        vbox.pack_start(hbox,0,0)
        hbox.show()
                
        for i in range(100):
            button = gtk.Button()
            hbox.pack_start(button,0,0)
            button.hide()
            button.connect("clicked", ActionGoTo)    
            self._children.append( button )

        #######################################################
            
        button = gtk.Button()
        button.show()
        button.connect("clicked",ActionGoTo)
        hbox.pack_end(button,0,0)
                    
        self._parent = button

        label = gtk.Label("")
        label.set_markup("<b>Parent:</b> ")
        label.show()
        hbox.pack_end(label,0,0)
            
        #######################################################

        window = gtk.ScrolledWindow()
        window.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
        window.show() 
        
        vbox.pack_start(window,1,1)
        tag_table = gtk.TextTagTable()

        tag_bold = gtk.TextTag("bold")
        tag_bold.set_property("weight", pango.WEIGHT_BOLD)

        tag_large = gtk.TextTag("large")
        tag_large.set_property("scale", 1.2)

        tag_normal = gtk.TextTag("normal")

        tag_table.add(tag_bold)
        tag_table.add(tag_large)

        buffer =  gtk.TextBuffer(tag_table)
        text = gtk.TextView(buffer)
        text.show()
        window.add(text)
        
        buffer.connect("changed",ActionSetDirty)


        self._content = buffer
        
        #######################################################
        # Buttons
        #######################################################

        hbox = gtk.HBox()
        vbox.pack_start(hbox,0,0)
        hbox.show()

        #######################################################
        
        def ActionUpdate(widget):
            global Global
            record = self._record
            newtag = self._tag.get_text()
            oldtag = record._tag
            if newtag != oldtag:
                # hack FIXME: needs to use zettel API
                del ClassZettel.Zettels[record._tag]
                ClassZettel.Zettels[newtag] = record
                record._tag = newtag
                if os.path.exists( Global._path + oldtag ):
                    os.rename(Global._path + oldtag,  Global._path + newtag)
                
            record._name =  self._title.get_text()

            if self._content.get_modified():
                fulltext = self._content.get_text(self._content.get_start_iter(),self._content.get_end_iter(),1)
                WriteTaggedBufferToFile(buffer, Global._path + record._tag )

            WriteDatabase(Global._database)
            self.edit_reset()
            return
        
        button = gtk.Button(" Update ")
        hbox.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionUpdate)    
        self._update_button = button

        #######################################################
        
        def ActionReset(widget):
            self.edit_reset()
            return
        
        button = gtk.Button(" Reset ")
        hbox.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionReset)    
        self._reset_button = button


        #######################################################

        def ActionBack(widget):
            if  self._dirty:
                 wutil.gui_message("Warning","Reset or Save Changes First",[("OK",None)])
            else:
                SwitchToView()
            return

        button = gtk.Button(" Back ")
        hbox.pack_end(button,0,0)
        button.show()
        button.connect("clicked", ActionBack)    

        #######################################################

        def ActionClickedNewEntry(widget):
            mark = buffer.get_insert()  
            iter = buffer.get_iter_at_mark(mark)
            buffer.insert_with_tags(iter,
                                    "==================================================\n" +
                                    get_current_time_string() + ": New Entry\n" +
                                    "==================================================\n",tag_large,tag_bold)
#            buffer.place_cursor(iter)

            buffer.move_mark(mark,iter)
            self.set_dirty(1) 
            return

        button = gtk.Button(" New Entry ")
        hbox.pack_end(button,0,0)
        button.show()
        button.connect("clicked", ActionClickedNewEntry)    

        #######################################################

#        label = gtk.Label("")
#        label.show()
#        hbox.pack_end(label,0,0)
#        self._dirty_label = label

        #######################################################
        vbox.show()
        self._widget = vbox
        return
      
    def get_widget(self):
        return self._widget

    def edit_reset(self):
        """ """
        record = self._record
        
        self._tag.set_text(record._tag)
        self._title.set_text(record._name)

        ReadTaggedBufferFromFile(self._content, Global._path + record._tag)

        self.set_dirty(0)
        self._content.set_modified(0)
        return
            
    def set_edit_record(self, record):
        self._record = record

        parent = record._parent
        if parent:
            self._parent.show()
            self._parent.set_label( parent._tag )
        else:
            self._parent.hide()

        for i in self._children:
            i.hide()

        for i in range(len(record._children)):
            child = record._children[i]
            self._children[i].set_label( child._tag )
            self._children[i].show()
                
        self.edit_reset()
        return
        


#######################################################
##
#######################################################

def CreateWidget(tooltips,filename = None):

    global Global

    Global._path =  wconfig.zettel.path
    
    if not filename:
          filename = wconfig.zettel.database

    Global._database = wdatabase.ClassDatabase()

    ReadDatabase(Global._database,filename)
    vbox = gtk.VBox()

    Global._edit = ClassEditZettel(tooltips)
    vbox.pack_start(Global._edit.get_widget())
    
    Global._tree_view = ClassTreeView(tooltips)
    vbox.pack_start(Global._tree_view.get_widget())
    
#    SwitchToEditor(Global._zettels[""])
    SwitchToView()

    
    vbox.show()

    return vbox

#######################################################
## 
#######################################################


if __name__ == '__main__':

    window = gtk.Window()

    def ActionDestroy(args):
        window.destroy()
        gtk.mainquit()

    window.connect("destroy", ActionDestroy)
    window.set_title('NOTES')
#    window.set_border_width(0)
#    window.set_usize(wconfig.win.width, wconfig.win.height)

    tooltips = gtk.Tooltips()

    if len(sys.argv) > 1:
        widget = CreateWidget(tooltips,sys.argv[1] )
    else:
        widget = CreateWidget(tooltips)

    tooltips.enable()


    window.add(widget)
    window.show()
    gtk.mainloop()

#######################################################
## eof
#######################################################







